@@ -3,6 +3,7 @@ from __future__ import annotations
 import datetime
 import re
 import typing as t
+from functools import partial
 
 from sqlglot import exp, generator, parser, tokens, transforms
 from sqlglot.dialects.dialect import (
@@ -211,7 +212,8 @@ def _string_agg_sql(self: TSQL.Generator, expression: exp.GroupConcat) -> str:
     if isinstance(expression.this, exp.Order):
         if expression.this.this:
             this = expression.this.this.pop()
-        order = f" WITHIN GROUP ({self.sql(expression.this)[1:]})"  # Order has a leading space
+        # Order has a leading space
+        order = f" WITHIN GROUP ({self.sql(expression.this)[1:]})"
 
     separator = expression.args.get("separator") or exp.Literal.string(",")
     return f"STRING_AGG({self.format_args(this, separator)}){order}"
@@ -451,11 +453,14 @@ class TSQL(Dialect):
             **tokens.Tokenizer.KEYWORDS,
             "DATETIME2": TokenType.DATETIME,
             "DATETIMEOFFSET": TokenType.TIMESTAMPTZ,
-            "DECLARE": TokenType.COMMAND,
+            "DECLARE": TokenType.DECLARE,
             "EXEC": TokenType.COMMAND,
+            "FOR SYSTEM_TIME": TokenType.TIMESTAMP_SNAPSHOT,
             "IMAGE": TokenType.IMAGE,
             "MONEY": TokenType.MONEY,
             "NTEXT": TokenType.TEXT,
+            "OPTION": TokenType.OPTION,
+            "OUTPUT": TokenType.RETURNING,
             "PRINT": TokenType.COMMAND,
             "PROC": TokenType.PROCEDURE,
             "REAL": TokenType.FLOAT,
@@ -463,17 +468,17 @@ class TSQL(Dialect):
             "SMALLDATETIME": TokenType.DATETIME,
             "SMALLMONEY": TokenType.SMALLMONEY,
             "SQL_VARIANT": TokenType.VARIANT,
+            "SYSTEM_USER": TokenType.CURRENT_USER,
             "TOP": TokenType.TOP,
             "TIMESTAMP": TokenType.ROWVERSION,
+            "TINYINT": TokenType.UTINYINT,
             "UNIQUEIDENTIFIER": TokenType.UNIQUEIDENTIFIER,
             "UPDATE STATISTICS": TokenType.COMMAND,
             "XML": TokenType.XML,
-            "OUTPUT": TokenType.RETURNING,
-            "SYSTEM_USER": TokenType.CURRENT_USER,
-            "FOR SYSTEM_TIME": TokenType.TIMESTAMP_SNAPSHOT,
-            "OPTION": TokenType.OPTION,
         }
 
+        COMMANDS = {*tokens.Tokenizer.COMMANDS, TokenType.END}
+
     class Parser(parser.Parser):
         SET_REQUIRES_ASSIGNMENT_DELIMITER = False
         LOG_DEFAULTS_TO_LN = True
@@ -526,7 +531,7 @@ class TSQL(Dialect):
 
         STATEMENT_PARSERS = {
             **parser.Parser.STATEMENT_PARSERS,
-            TokenType.END: lambda self: self._parse_command(),
+            TokenType.DECLARE: lambda self: self._parse_declare(),
         }
 
         def _parse_options(self) -> t.Optional[t.List[exp.Expression]]:
@@ -711,6 +716,32 @@ class TSQL(Dialect):
 
             return partition
 
+        def _parse_declare(self) -> exp.Declare | exp.Command:
+            index = self._index
+            expressions = self._try_parse(partial(self._parse_csv, self._parse_declareitem))
+
+            if not expressions or self._curr:
+                self._retreat(index)
+                return self._parse_as_command(self._prev)
+
+            return self.expression(exp.Declare, expressions=expressions)
+
+        def _parse_declareitem(self) -> t.Optional[exp.DeclareItem]:
+            var = self._parse_id_var()
+            if not var:
+                return None
+
+            value = None
+            self._match(TokenType.ALIAS)
+            if self._match(TokenType.TABLE):
+                data_type = self._parse_schema()
+            else:
+                data_type = self._parse_types()
+                if self._match(TokenType.EQ):
+                    value = self._parse_bitwise()
+
+            return self.expression(exp.DeclareItem, this=var, kind=data_type, default=value)
+
     class Generator(generator.Generator):
         LIMIT_IS_TOP = True
         QUERY_HINTS = False
@@ -753,11 +784,12 @@ class TSQL(Dialect):
             exp.DataType.Type.DATETIME: "DATETIME2",
             exp.DataType.Type.DOUBLE: "FLOAT",
             exp.DataType.Type.INT: "INTEGER",
+            exp.DataType.Type.ROWVERSION: "ROWVERSION",
             exp.DataType.Type.TEXT: "VARCHAR(MAX)",
             exp.DataType.Type.TIMESTAMP: "DATETIME2",
             exp.DataType.Type.TIMESTAMPTZ: "DATETIMEOFFSET",
+            exp.DataType.Type.UTINYINT: "TINYINT",
             exp.DataType.Type.VARIANT: "SQL_VARIANT",
-            exp.DataType.Type.ROWVERSION: "ROWVERSION",
         }
 
         TYPE_MAPPING.pop(exp.DataType.Type.NCHAR)
@@ -920,6 +952,10 @@ class TSQL(Dialect):
         def create_sql(self, expression: exp.Create) -> str:
             kind = expression.kind
             exists = expression.args.pop("exists", None)
+
+            if kind == "VIEW":
+                expression.this.set("catalog", None)
+
             sql = super().create_sql(expression)
 
             like_property = expression.find(exp.LikeProperty)
@@ -1061,3 +1097,22 @@ class TSQL(Dialect):
             if isinstance(action, exp.RenameTable):
                 return f"EXEC sp_rename '{self.sql(expression.this)}', '{action.this.name}'"
             return super().altertable_sql(expression)
+
+        def drop_sql(self, expression: exp.Drop) -> str:
+            if expression.args["kind"] == "VIEW":
+                expression.this.set("catalog", None)
+            return super().drop_sql(expression)
+
+        def declare_sql(self, expression: exp.Declare) -> str:
+            return f"DECLARE {self.expressions(expression, flat=True)}"
+
+        def declareitem_sql(self, expression: exp.DeclareItem) -> str:
+            variable = self.sql(expression, "this")
+            default = self.sql(expression, "default")
+            default = f" = {default}" if default else ""
+
+            kind = self.sql(expression, "kind")
+            if isinstance(expression.args.get("kind"), exp.Schema):
+                kind = f"TABLE {kind}"
+
+            return f"{variable} AS {kind}{default}"
