@@ -7,6 +7,7 @@ from sqlglot import alias, exp
 from sqlglot.dialects.dialect import Dialect, DialectType
 from sqlglot.errors import OptimizeError
 from sqlglot.helper import seq_get, SingleValuedMapping
+from sqlglot.optimizer.annotate_types import TypeAnnotator
 from sqlglot.optimizer.scope import Scope, build_scope, traverse_scope, walk_in_scope
 from sqlglot.optimizer.simplify import simplify_parens
 from sqlglot.schema import Schema, ensure_schema
@@ -48,8 +49,10 @@ def qualify_columns(
         - Currently only handles a single PIVOT or UNPIVOT operator
     """
     schema = ensure_schema(schema)
+    annotator = TypeAnnotator(schema)
     infer_schema = schema.empty if infer_schema is None else infer_schema
-    pseudocolumns = Dialect.get_or_raise(schema.dialect).PSEUDOCOLUMNS
+    dialect = Dialect.get_or_raise(schema.dialect)
+    pseudocolumns = dialect.PSEUDOCOLUMNS
 
     for scope in traverse_scope(expression):
         resolver = Resolver(scope, schema, infer_schema=infer_schema)
@@ -73,6 +76,9 @@ def qualify_columns(
         _expand_group_by(scope)
         _expand_order_by(scope, resolver)
 
+        if dialect == "bigquery":
+            annotator.annotate_scope(scope)
+
     return expression
 
 
@@ -209,7 +215,7 @@ def _expand_alias_refs(scope: Scope, resolver: Resolver) -> None:
         if not node:
             return
 
-        for column, *_ in walk_in_scope(node, prune=lambda node, *_: node.is_star):
+        for column in walk_in_scope(node, prune=lambda node: node.is_star):
             if not isinstance(column, exp.Column):
                 continue
 
@@ -306,7 +312,7 @@ def _expand_positional_references(
             else:
                 select = select.this
 
-                if isinstance(select, exp.Literal):
+                if isinstance(select, exp.CONSTANTS) or select.find(exp.Explode, exp.Unnest):
                     new_nodes.append(node)
                 else:
                     new_nodes.append(select.copy())
@@ -425,7 +431,7 @@ def _expand_stars(
                 raise OptimizeError(f"Unknown table: {table}")
 
             columns = resolver.get_source_columns(table, only_visible=True)
-            columns = columns or scope.outer_column_list
+            columns = columns or scope.outer_columns
 
             if pseudocolumns:
                 columns = [name for name in columns if name.upper() not in pseudocolumns]
@@ -517,7 +523,7 @@ def qualify_outputs(scope_or_expression: Scope | exp.Expression) -> None:
 
     new_selections = []
     for i, (selection, aliased_column) in enumerate(
-        itertools.zip_longest(scope.expression.selects, scope.outer_column_list)
+        itertools.zip_longest(scope.expression.selects, scope.outer_columns)
     ):
         if selection is None:
             break
@@ -544,7 +550,7 @@ def quote_identifiers(expression: E, dialect: DialectType = None, identify: bool
     """Makes sure all identifiers that need to be quoted are quoted."""
     return expression.transform(
         Dialect.get_or_raise(dialect).quote_identifier, identify=identify, copy=False
-    )
+    )  # type: ignore
 
 
 def pushdown_cte_alias_columns(expression: exp.Expression) -> exp.Expression:
@@ -652,8 +658,16 @@ class Resolver:
 
         if isinstance(source, exp.Table):
             columns = self.schema.column_names(source, only_visible)
-        elif isinstance(source, Scope) and isinstance(source.expression, exp.Values):
-            columns = source.expression.alias_column_names
+        elif isinstance(source, Scope) and isinstance(source.expression, (exp.Values, exp.Unnest)):
+            columns = source.expression.named_selects
+
+            # in bigquery, unnest structs are automatically scoped as tables, so you can
+            # directly select a struct field in a query.
+            # this handles the case where the unnest is statically defined.
+            if self.schema.dialect == "bigquery":
+                if source.expression.is_type(exp.DataType.Type.STRUCT):
+                    for k in source.expression.type.expressions:  # type: ignore
+                        columns.append(k.name)
         else:
             columns = source.expression.named_selects
 
