@@ -300,6 +300,10 @@ class TestBuild(unittest.TestCase):
                 lambda: select("x").from_("tbl").order_by("y"),
                 "SELECT x FROM tbl ORDER BY y",
             ),
+            (
+                lambda: parse_one("select * from x union select * from y").order_by("y"),
+                "SELECT * FROM x UNION SELECT * FROM y ORDER BY y",
+            ),
             (
                 lambda: select("x").from_("tbl").cluster_by("y"),
                 "SELECT x FROM tbl CLUSTER BY y",
@@ -505,15 +509,19 @@ class TestBuild(unittest.TestCase):
             (lambda: parse_one("(SELECT 1)").select("2"), "(SELECT 1, 2)"),
             (
                 lambda: parse_one("(SELECT 1)").limit(1),
-                "SELECT * FROM ((SELECT 1)) AS _l_0 LIMIT 1",
+                "(SELECT 1) LIMIT 1",
             ),
             (
                 lambda: parse_one("WITH t AS (SELECT 1) (SELECT 1)").limit(1),
-                "SELECT * FROM (WITH t AS (SELECT 1) (SELECT 1)) AS _l_0 LIMIT 1",
+                "WITH t AS (SELECT 1) (SELECT 1) LIMIT 1",
             ),
             (
                 lambda: parse_one("(SELECT 1 LIMIT 2)").limit(1),
-                "SELECT * FROM ((SELECT 1 LIMIT 2)) AS _l_0 LIMIT 1",
+                "(SELECT 1 LIMIT 2) LIMIT 1",
+            ),
+            (
+                lambda: parse_one("SELECT 1 UNION SELECT 2").limit(5).offset(2),
+                "SELECT 1 UNION SELECT 2 LIMIT 5 OFFSET 2",
             ),
             (lambda: parse_one("(SELECT 1)").subquery(), "((SELECT 1))"),
             (lambda: parse_one("(SELECT 1)").subquery("alias"), "((SELECT 1)) AS alias"),
@@ -537,6 +545,10 @@ class TestBuild(unittest.TestCase):
                 lambda: exp.update("tbl", {"x": 1}, from_="tbl2"),
                 "UPDATE tbl SET x = 1 FROM tbl2",
             ),
+            (
+                lambda: exp.update("tbl", {"x": 1}, from_="tbl2 cross join tbl3"),
+                "UPDATE tbl SET x = 1 FROM tbl2 CROSS JOIN tbl3",
+            ),
             (
                 lambda: union("SELECT * FROM foo", "SELECT * FROM bla"),
                 "SELECT * FROM foo UNION SELECT * FROM bla",
@@ -665,14 +677,8 @@ class TestBuild(unittest.TestCase):
                 "(x, y) IN ((1, 2), (3, 4))",
                 "postgres",
             ),
-            (
-                lambda: exp.cast_unless("CAST(x AS INT)", "int", "int"),
-                "CAST(x AS INT)",
-            ),
-            (
-                lambda: exp.cast_unless("CAST(x AS TEXT)", "int", "int"),
-                "CAST(CAST(x AS TEXT) AS INT)",
-            ),
+            (lambda: exp.cast("CAST(x AS INT)", "int"), "CAST(x AS INT)"),
+            (lambda: exp.cast("CAST(x AS TEXT)", "int"), "CAST(CAST(x AS TEXT) AS INT)"),
             (
                 lambda: exp.rename_column("table1", "c1", "c2", True),
                 "ALTER TABLE table1 RENAME COLUMN IF EXISTS c1 TO c2",
